
import re
from model_training import SentimentModel
from sentence import Sentence
 
class Business(object):
    """
    用来表示跟business相关的变量和函数
    """

    SENTIMENT_MODEL = SentimentModel() # 把已经训练好的模型存放在文件里，并导入进来


    def __init__(self, review_df):
        # 初始化变量以及函数
        self.reviews = review_df

    def aspect_based_summary(self,biz_id,SENTIMENT_MODEL):
        """
        返回一个business的summary. 针对于每一个aspect计算出它的正面负面情感以及TOP reviews. 
        具体细节请看给定的文档。 
        """
        
        
        review = self.reviews[self.reviews.business_id == biz_id] # extract business id reviews
        business_name = review[review.business_id == biz_id].name.unique()[0] # extract business name
        business_rating = review[review.business_id == biz_id].stars.sum()/len(review) # overall business rating
        
        biz_info = dict()
        biz_info['business_id'] = biz_id
        biz_info['business_name'] = business_name
        biz_info['business_rating'] = business_rating
        
        
        pos = dict()
        neg = dict()
        main_aspects = self.main_aspect_extract(biz_id,review)

        for i in main_aspects.keys():
            aspect = list(main_aspects[i].keys())

            for j in range(5):
                # create empty dictionary for 5 aspects
                biz_info[aspect[j]] = {}
                biz_info[aspect[j]]['rating'] = []
                biz_info[aspect[j]]['pos'] = {}
                biz_info[aspect[j]]['neg'] = {}


            for stars,rw in review[['stars','text']].values:
                rw = rw.lower()
                
                # following is to predict reviews that contains aspects and calculating rating for correponding aspects
                if aspect[0] in rw:
                    sent_pred, sent_score_pred = SENTIMENT_MODEL.predict([rw])
                    if (sent_pred[0] == 'positive') & (sent_score_pred[0][2] > 0.7):
                          biz_info[aspect[0]]['pos'][rw] = sent_score_pred[0][2] 
                    elif (sent_pred[0] == 'negative') & (sent_score_pred[0][0] > 0.7):
                        biz_info[aspect[0]]['neg'][rw] = sent_score_pred[0][0] 
                    else:
#                         print('neutral: do nothing')
                          None
                    biz_info[aspect[0]]['rating'].append(stars)
                    
                    continue
                
                elif aspect[1] in rw:
                    sent_pred, sent_score_pred = SENTIMENT_MODEL.predict([rw])
                    if (sent_pred[0] == 'positive') & (sent_score_pred[0][2] > 0.7):
                        biz_info[aspect[1]]['pos'][rw] = sent_score_pred[0][2]
                    elif (sent_pred[0] == 'negative') & (sent_score_pred[0][0] > 0.7):
                        biz_info[aspect[1]]['neg'][rw] = sent_score_pred[0][0]
                    else:
#                         print('neutral: do nothing')
                          None

                    biz_info[aspect[1]]['rating'].append(stars)
                    
                    continue
                    
                elif aspect[2] in rw:
                    sent_pred, sent_score_pred = SENTIMENT_MODEL.predict([rw])
                    if (sent_pred[0] == 'positive') & (sent_score_pred[0][2] > 0.7):
                        biz_info[aspect[2]]['pos'][rw] = sent_score_pred[0][2]
                    elif (sent_pred[0] == 'negative') & (sent_score_pred[0][0] > 0.7):
                        biz_info[aspect[2]]['neg'][rw] = sent_score_pred[0][0]
                    else:
#                         print('neutral: do nothing')
                          None

                    biz_info[aspect[2]]['rating'].append(stars)
                    
                    continue
                    
                elif aspect[3] in rw:
                    sent_pred, sent_score_pred = SENTIMENT_MODEL.predict([rw])
                    if (sent_pred[0] == 'positive') & (sent_score_pred[0][2] > 0.7):
                        biz_info[aspect[3]]['pos'][rw] = sent_score_pred[0][2]
                    elif (sent_pred[0] == 'negative') & (sent_score_pred[0][0] > 0.7):
                        biz_info[aspect[3]]['neg'][rw] = sent_score_pred[0][0]
                    else:
#                         print('neutral: do nothing')
                          None
                    biz_info[aspect[3]]['rating'].append(stars)
                    
                    continue
                    
                elif aspect[4] in rw:
                    
                    sent_pred, sent_score_pred = SENTIMENT_MODEL.predict([rw])
                    if (sent_pred[0] == 'positive') & (sent_score_pred[0][2] > 0.7):
                        biz_info[aspect[4]]['pos'][rw] = sent_score_pred[0][2]
                    elif (sent_pred[0] == 'negative') & (sent_score_pred[0][0] > 0.7):
                        biz_info[aspect[4]]['neg'][rw] = sent_score_pred[0][0]
                    else:
#                         print('neutral: do nothing')
                          None
                    biz_info[aspect[4]]['rating'].append(stars)
                    continue


            for j in range(5):
                # loop through all 5 aspects dictionary and pick top 5 reviews that has highest prediction score (from sentiment model)
                biz_info[aspect[j]]['pos'] = dict(sorted(biz_info[aspect[j]]['pos'].items(), key=lambda item: item[1],reverse=True)[:5])
                biz_info[aspect[j]]['neg'] = dict(sorted(biz_info[aspect[j]]['neg'].items(), key=lambda item: item[1],reverse=True)[:5])
                biz_info[aspect[j]]['rating'] = round(sum(biz_info[aspect[j]]['rating'])/len(biz_info[aspect[j]]['rating']) if len(biz_info[aspect[j]]['rating']) else 0,2)


        return biz_info

    def main_aspect_extract(self,biz_id,review):
        """
        从一个business的review中抽取aspects
        """
        
        bus_aspect = dict()
        sent = Sentence()
        clean_words = sent.clean_symbols(review.text.to_list()) # clean symbol
        clean_words = sent.extract_aspect(clean_words,sent) # extract all aspects (nouns words)
        clean_words = sent.words_cnt(clean_words) # count the frequency of each aspects (nouns words)
        aspects = dict(sorted(clean_words.items(), key=lambda item: item[1],reverse=True)[:5]) # pick top five aspects
        bus_aspect[biz_id] = aspects
        
        return bus_aspect




