import time
import pandas as pd
from model_training import SentimentModel
from business import Business

def processed_review():
    
    # dataset name
    review = 'data/review.json'
    business_json_path = 'data/business.json'

    # load dataset
    df_r = pd.read_json(review, lines=True)
    df_b = pd.read_json(business_json_path, lines=True)

    # count review of each business
    review_cnt = df_r.groupby('business_id').agg({'review_id':pd.Series.nunique})
    review_cnt = review_cnt.reset_index()
    
    # filter out business that has reviews less than 100
    filter_review_cnt = review_cnt[review_cnt.review_id > 100]
    df_filter_review = df_r[df_r.business_id.isin(filter_review_cnt.business_id)]

    #  get business name
    df_reviews = pd.merge(df_filter_review,df_b[['business_id','name']],on='business_id')
    return df_reviews

def get_review_summary_for_business(biz_id,reviews):
    # 获取每一个business的评论总结

    sm = SentimentModel() # initialise SentimentModel class
    business = Business(reviews) # initialise Business class
    aspect_sum = business.aspect_based_summary(biz_id,sm) # get aspect summary

    return aspect_sum
   
    
def main(): 



#     reviews = processed_review() # uncomment this to process the raw reviews data
    df_reviews = pd.read_csv('data/cleaned_reviews.csv') # results of previous function


    bus_ids = ['ujmEBvifdJM6h6RLv4wQIg','eU_713ec6fTGNO4BegRaww'] # 指定几个business ids
    
    for bus_id in bus_ids:
        print('\n')
        print ("Working on biz_id %s" % bus_id)
#         start = time.time()
        summary = get_review_summary_for_business(bus_id,df_reviews)
        i = 0
        for key, value in summary.items():
            if i < 3:
                print(key,':',value)
                i += 1
            else:
                if i == 3:
                    j = 1
                    i += 1
                    
                print(f'Aspect {j}')
                print(key,':',value)
                j += 1
        # format and print....

if __name__ == "__main__":
    main()